#include <iostream>
#include "list.h"

using namespace std;

// Konstruktor: létrehozza a láncolt lista fejelemét és beállítja a mutatókat.
List::List()
{
	header = new Node();
	last = header;
}

// Copy konstruktor: lemásolja az argumentumban megadott objektumot.
List::List(const List &ol)
{
	header = new Node(ol.header->value);
	last = header;
	
	Node *current = ol.header->next;
	while(current != NULL)
	{
		append(current->value);
		current = current->next;
	}
}

// Destruktor, a HEAP-en lefoglalt terület felszabadítása.
List::~List()
{
	Node *current;
	while(header != NULL)
	{
		current = header;
		header = header->next;
		delete current;
	}
}

// Értékadás operátor: lemásolja az argumentumban megadott objektumot, ha nem saját magát adtuk értékül.
List &List::operator=(const List &ol)
{
	// Saját magának próbáljuk-e értékül adni?
	if(this == &ol)
		return *this;

	// Régi lista megsemmisítése.
	Node *current;
	while(header != NULL)
	{
		current = header;
		header = header->next;
		delete current;
	}

	// Paraméter lista átmásolása.
	header = new Node(ol.header->value);
	last = header;
	
	current = ol.header->next;
	while(current != NULL)
	{
		append(current->value);
		current = current->next;
	}
	
	return *this;
}

// Elem hozzáfűzése a láncolt lista végére.
void List::append(int v)
{
	last->next = new Node(v);
	last = last->next;
}

// Elem befűzése a láncolt lista elejére.
void List::prepend(int v)
{
	header->next = new Node(v, header->next);
}

// Elem kivétele a láncolt lista elejéről. Ha a lista üres, akkor kivétel kiváltása.
int List::pop() throw(Exception)
{
	if(header->next == NULL)
		throw Empty;
	
	Node* first = header->next;
	int result = first->value;
	
	header->next = first->next;
	delete first;	
	return result;
}

// A láncolt listában tárolt maximális érték megkeresése. Ha a lista üres, akkor kivétel kiváltása.
int List::getMax() const throw(Exception)
{
	if(header->next == NULL)
		throw Empty;
	
	Node *max = header->next;
	Node *current = max->next;
	while(current != NULL)
	{
		if(current->value > max->value)
			max = current;
		current = current->next;
	}
	return max->value;
}

// Kiíró operátor megvalósítása, a láncolt lista elemeinek felsorolása.
std::ostream &operator<<(std::ostream &out, const List &l)
{
	List::Node *current = l.header->next;
	while(current != NULL)
	{
		out << current->value << " ";
		current = current->next;
	}
	out << endl;
	return out;
}
