#ifndef LIST_H
#define LIST_H

#include <iostream>

// Egyirányú, fejelemes láncolt lista.
class List
{
	// Publikus rész (implementáció).
	public:
		// Kivételek.
		enum Exception
		{
			// Azt az esetet jelöli, amikor a lista üres, de az adott művelet így nem értelmezhető.
			Empty
		};

		// Konstruktor: létrehozza a láncolt lista fejelemét és beállítja a mutatókat.
		List();
		// Copy konstruktor: lemásolja az argumentumban megadott objektumot.
		List(const List &l);
		// Destruktor, a HEAP-en lefoglalt terület felszabadítása.
		virtual ~List();
		// Értékadás operátor: lemásolja az argumentumban megadott objektumot, ha nem saját magát adtuk értékül.
		List &operator=(const List &ol);
		
		// Elem hozzáfűzése a láncolt lista végére.
		void append(int v);
		// Elem befűzése a láncolt lista elejére.
		void prepend(int v);
		// Elem kivétele a láncolt lista elejéről. Ha a lista üres, akkor kivétel kiváltása.
		int pop() throw(Exception);
		// A láncolt listában tárolt maximális érték megkeresése. Ha a lista üres, akkor kivétel kiváltása.
		int getMax() const throw(Exception);
		
		// Kiíró operátor megvalósítása, a láncolt lista elemeinek felsorolása.
		friend std::ostream &operator<<(std::ostream &out, const List &l);
		
	// Privát rész (reprezentáció).
	private:
		// A láncolt lista elemeinek reprezentációja.
		struct Node
		{
			// Tárolt érték.
			int value;
			// Következő elem mutatója.
			Node* next;
			
			// Konstruktor: létrehoz egy új láncolt lista elemet.
			Node(int v = 0, Node *n = NULL) : value(v), next(n) {}
		};
		
		// Fejmutató és az utolsó elem mutatója.
		Node *header, *last;
	};

#endif
